# vim: fileencoding=UTF-8
import sys;

def usage():
	print(\
"""Usage : python typegen.py classname valuename m kg s A K mol cd
abstruct:
  This python script generates a template source code for dimension promise library.

example:
  $ python typegen.py Length meter 1 0 0 0 0 0 0
  It generates a scalar type "Length".

arguments:
  classname : C++ class name. Generates a template source code for the class.
  valuename : The parameter name of a constructor.
  m, kg, s, A, K, mol, cd
            : SI Units of the class.
""")

if len(sys.argv) >= 2 and sys.argv[1] == '--help' or len(sys.argv) == 1:
	usage()
	exit(1)

if len(sys.argv) < 10:
	print('invalid option(s)')
	usage()
	exit(1)

classname = sys.argv[1];
valuename = sys.argv[2];
basename = classname + 'Base'
traitsname = basename + 'Traits'
dim = list()

for i in range(0, 7):
	dim.append(sys.argv[3 + i])

dimstr = ''
for d in dim:
	dimstr = dimstr + ', ' + d

print('#ifndef DIMPRO_' + classname.upper() + '_HPP_')
print('#define DIMPRO_' + classname.upper() + '_HPP_')
print('')
print('#include "scalar.hpp"')
print('')
print('namespace NSDimensionPromise')
print('{')
print('')
print('/**')
print(' * ' + classname + 'の次元定義.')
print(' */')
print('template <typename T>')
print('struct ' + traitsname)
print('{')
print('  typedef ScalarNumber<T' + dimstr + '> ScalarType;')
print('};')
print('')
print('/**')
print(' * ' + classname + 'を表すベースクラス.')
print(' */')
print('template <typename T>')
print('class ' + basename + ' : public ' + traitsname + '<T>::ScalarType')
print('{')
print('public:')
print('  /**')
print('   * ' + classname + 'を表すスカラー変数を生成する.')
print('   * @param ' + valuename + ' 変数の初期値')
print('   */')
print('  explicit ' + basename + '(const T& ' + valuename + ' = T())')
print('  : ' + traitsname + '<T>::ScalarType(' + valuename + ')')
print('  {  }')
print('')
print('  /**')
print('   * ' + classname + 'を表すスカラー変数をコピーして生成する.')
print('   * @param rhs コピー元の変数')
print('   */')
print('  ' + basename + '(const typename ' + traitsname + '<T>::ScalarType& rhs)')
print('  : ' + traitsname + '<T>::ScalarType(rhs)')
print('  {  }')
print('')
print('  /**')
print('    * ' + valuename + '単位の値を得る.')
print('    * @return ' + valuename)
print('    */')
print('  const T& To' + valuename[0].upper() + valuename[1:] + '() const')
print('  {')
print('    return this->X();')
print('  }')
print('')
print('  /**')
print('    * hoge単位の値を得る.')
print('    * @return hoge')
print('    */')
print('  const T ToHoge() const')
print('  {')
print('    return static_cast<T>(this->X() * hoge);')
print('  }')
print('')
print('  /**')
print('    * ' + valuename + 'から' + classname + 'を表すスカラー変数を生成する.')
print('    * @param ' + valuename + ' ' + valuename)
print('    * @return ' + classname + 'を表すスカラー変数')
print('    */')
print('  static ' + basename + ' From' + valuename[0].upper() + valuename[1:] + '(const T& ' + valuename + ')')
print('  {')
print('    return ' + basename + '(' + valuename + ');')
print('  }')
print('')
print('  /**')
print('    * hogeから' + classname + 'を表すスカラー変数を生成する.')
print('    * @param hoge hoge')
print('    * @return ' + classname + 'を表すスカラー変数')
print('    */')
print('  static ' + basename + ' FromHoge(const T& hoge)')
print('  {')
print('    return ' + basename + '(hoge);')
print('  }')
print('};')
print('')
print('/**')
print(' * よく使う' + classname + 'の型.')
print(' */')
print('typedef ' + basename + '<NormalValueType> ' + classname + ';')
print('')
print('}')
print('')
print('#endif /* DIMPRO_' + classname.upper() + '_HPP_ */')

