/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Length.hpp
 *
 *  Created on: 2010/11/18
 *      Author: HARK Development Team
 */

#ifndef DIMPRO_LENGTH_HPP_
#define DIMPRO_LENGTH_HPP_

#include "scalar.hpp"

namespace NSDimensionPromise
{

/**
 * Lengthの次元定義.
 */
template <typename T>
struct LengthBaseTraits
{
    typedef ScalarNumber<T, 1, 0, 0, 0, 0, 0, 0> ScalarType;
};

/**
 * Lengthを表すベースクラス.
 */
template <typename T>
class LengthBase : public LengthBaseTraits<T>::ScalarType
{
public:
    /**
     * Lengthを表すスカラー変数を生成する.
     * @param meter 変数の初期値
     */
    explicit LengthBase(const T& meter = T())
        : LengthBaseTraits<T>::ScalarType(meter)
    {  }

    /**
     * Lengthを表すスカラー変数をコピーして生成する.
     * @param rhs コピー元の変数
     */
    LengthBase(const typename LengthBaseTraits<T>::ScalarType& rhs)
        : LengthBaseTraits<T>::ScalarType(rhs)
    {  }

    /**
      * メートル単位の値を得る.
      * @return メートル
      */
    const T& ToMeter() const
    {
        return this->X();
    }

    /**
      * ミリメートル単位の値を得る.
      * @return ミリメートル
      */
    const T ToMillimeter() const
    {
        return static_cast<T>(this->X() * 1e3);
    }

    /**
      * メートルからLengthを表すスカラー変数を生成する.
      * @return Lengthを表すスカラー変数
      */
    static LengthBase FromMeter(const T& meter)
    {
        return LengthBase(meter);
    }

    /**
      * ミリメートルからLengthを表すスカラー変数を生成する.
      * @return Lengthを表すスカラー変数
      */
    static LengthBase FromMillimeter(const T& millimeter)
    {
        return LengthBase(millimeter * 1e-3);
    }
};

/**
 * よく使うLengthの型.
 */
typedef LengthBase<NormalValueType> Length;

}

#endif /* DIMPRO_LENGTH_HPP_ */
