/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * scalar.hpp
 *
 *  Created on: 2010/11/18
 *      Author: HARK Development Team
 */

#ifndef DIMPRO_SCALAR_HPP_
#define DIMPRO_SCALAR_HPP_

namespace NSDimensionPromise
{

/**
 * スカラーを表す型のベースクラス.
 * 継承することを前提にしたクラスであるが、実行時オーバーヘッドを減らすために
 * 仮想デストラクタを実装していない。
 * 派生クラスの中でデストラクタを使った場合、正しく呼び出されないので注意。
 * あくまで、doubleやfloatの単純なラッパとしての用途のみ想定されている。
 */
template <typename T, int m, int kg, int s, int A, int K, int mol, int cd>
class ScalarNumber
{
public:
    /**
     * スカラー変数を生成する.
     * @param x 変数の初期値
     */
    explicit ScalarNumber(const T& x = T())
        : x(x)
    { }

    /**
     * スカラーの値を得る.
     * @return スカラーの値
     */
    const T& X() const
    {
        return x;
    }
private:
    /**
     * スカラーの値.
     */
    T x;
};

/**
 * 一般的にスカラー値の格納に使う型.
 */
typedef double NormalValueType;

}

#endif /* DIMPRO_SCALAR_HPP_ */
